; ROUTINE TO GET A FILE SPECIFIER
;
; Function:
;       This routine gets a file specifier from the unformated parameter
;       area at 80h of the Program Segment Prefix and transfers it to an
;       ASCIIZ buffer. When you enter a transient command, the DOS auto-
;       matically fills the unformated parameter area with the tail of
;       the command line.
; Input:
;       On entry the unformatted parameter area must contain a file
;       specifer and DS:DX must point to a buffer in memory
; Output:
;       The buffer is filled with the file specification as an ASCIIZ string.
; Registers used:
;       DX used for input.
; Segments referenced:
;       On entry the extra segment must contain the ASCIIZ string buffer.
; Special notes:
;       The double word variable DTA must point to the default DTA in the
;       Program Segment Prefix. In particular the second word of this variable
;       should be loaded with th Program Segment Prefix when the user first
;       gains control. This segment is intially contained in DS.
;
getspec proc    far
;
        push    ds              ; save registers
        push    es
        push    si
        push    di
        push    cx
;
; set up pointer to dta to get parameters
        lds     si,dta          ; point to dta for parameters
        mov     cl,[si]         ; get length of string
        mov     ch,0            ; make a 16-bit
        inc     si              ; skip the length byte
; scan past the spaces
        mov     al,' '          ; skip spaces
getspec1:
        cmp     [si],al         ; check for space
        jne     getspec2        ; exit the loop if nonspace
        inc     si              ; other wise point to next byte
        loop    getspec1        ; loop back for more
        jcxz    getspec3        ; no file specifier ?
;
; move the rest in place
getspec2:
        mov     di,dx           ; index points to destination
        cld                     ; forward direction
        rep     movsb           ; make the transfer
        clc                     ; no error so no carry
        jmp     getspecexit     ; and return
;
getspec3:
        mov     ax,20           ; no file specified
        stc                     ; set carry for error
        jmp     getspecexit     ; and exit
;
getspecexit:
        pop     cx              ; restore registers
        pop     di
        pop     si
        pop     es
        pop     ds
        ret
;
getspec endp
;
