; ROUTINE TO HANDLE INPUT FOR CIRCULAR BUFFER
;
; Function:
;       This routine handles input to a circular buffer. It checks to
;       see if there is any input from a serial communications line. If there
;       is input then it is sent to a circular buffer. I a control Z is recieved
;       then carry is set and an exception code is returned. If there is no
;       input then it returns with the carry clear and without further action.
;  Input:
;       On entry DX specifies the communications line (0=com1: and 1=com2:)
;  Output:
;       The input characters go to a circular buffer.
; Registers used:
;       DI is the buffer pointer
;  Segments referenced:
;       On entry the data segment must contain the circular buffer and
;       the variables CBUFF, GAUGE, & LASTBYTE
;  Routines called:     COMON,COMOFF,COMINCK,PUTCBUFF
;
incbuff proc    far
;
; check for a character
        call cominck            ; check for a character
        jz      incbuffexit     ; good exit if none
;
; strip off parity bit
        and     al,7Fh          ; just 7 bits
;
; check for line feed
        cmp     al,10           ; ASCII 10 is line feed
        jne     incbuff1        ; skip if not line feed
        cmp     lastbyte,13     ; was last byte a carraige return ?
        je      incbuff2        ; skip if cr/lf
;
incbuff1:
        call    putcbuff        ; put byte into buffer
        mov     lastbyte,al     ; update last byte
;
; check for carraige returnand
incbuff2:
        cmp     al,13           ; ASCII 13 is carraige return
        jne     incbuff3        ; skip if not
;
; also put a linefeed if character was a carriage return
        push    ax              ; save current character
        mov     al,10           ; put in a linefeed
        call    putcbuff        ; into buffer
        pop     ax              ; restore current character
;
; check for end of file
incbuff3:
        cmp     al,controlz     ; end of file ?
        je      incbuff5        ; is an exception
;
; check for overflow
        cmp     guage,nearfull  ; too many characters ?
        jl      incbuff4        ; skip if not
        call    comoff          ; request to stop flow
        jmp     incbuffexit     ; good exit
;
; if not overflow make sure flow is on
incbuff4:
        call    comon           ; make sure flow is on
        jmp     incbuffexit     ; good exit
;
; handle end of file exception
incbuff5:
        mov     ax,19           ;end of file code
        stc                     ; set carry for error
        jmp     incbuffret      ; return with error
;
incbuffexit:
        clc                     ; clear carry - no error
;
incbuffret:
        ret
;
incbuff endp
;
