; ROUTINE TO SEARCH FOR A WORD IN AN ORDERED LIST OF WORDS
;
; Input:
;	On entry DS:BX points to a source word string.
;	ES:BP points to the ordered list of words
;	The source word string starts with a 16-bit integer which is it's length
;	The last byte must be a carriage return symbol (ASCII 13)
;	The desination list begins with a 16-bit integer specifing the length
;	in character bytes. It continues with words of ASCII characters
;	separated by carriage return symbols.
; Output:
;	On exit AL contains flag (0=not found, 0FFh=found)
;	ES:DX contains p;ace to insert the new word.
;	ES:DX points to location of this word, if present, in destination.
; Registers used:
;	AX & DX are modified and used for output
; Segments referenced:
;	On entry the data segment contains the source, and the extra segment
;	contains the destination word list
;
lexsearch	proc	far
;
	push	si		; save registers
	push	di
	push	cx
;
; point to beginning of list and get its length
	mov	di,bp		; beginning of text
	mov	cx,[di] ; get length
	inc	di
	inc	di
;
; compare source word with words in the list
;
lexsearch1:
;
	mov	dx,di		; save beginning of dest word
;
; forward direction
	cld			; forward direction
;
; point to beginning of source
	mov	si,bx		; point to beginning of source
	inc	si
	inc	si
;
; compare source word with a word with a word of the list
lexsearch2:
;
; check for end of list
	jcxz	lexsearch5	; end of list - insert it
;
; set up a carriage return as character for scanning
	mov	al,13		; scan for carriage return
;
; check for end of source word
	cmp	[si],al 	; source byte = carraige return ?
	je	lexsearch4	; end of source word found
;
;check for end of destination word
	cmp	es:[di],al	; dest byte = carraige return ?
	je	lexsearch3	; no match - go to next word
;
; compare character by character
	dec	cx
	cmpsb			; check for match
	je	lexsearch2	; matched - check next character
	jb	lexsearch5	; too high - this is the place
;
; scan for next carraige return
lexsearch3:
	repnz	scasb		; scan until next carraige return
	jmp	lexsearch1	; next word
;
; end of source word was found
lexsearch4:
	cmp	[di],al 	; dest character = carraige return ?
	je	lexsearch6	; end of destination word ?
;
; found a spot to insert the word
lexsearch5:
	mov	al,0FFh 	; success
	jmp	lexsearchexit
;
; word is already present
lexsearch6:
	mov	al,00h		; already there
	jmp	lexsearchexit
;
lexsearchexit:
	pop	cx		; restore registers
	pop	di
	pop	si
	ret
;
lexsearch	endp
;
