; ROUTINE TO FILL A RECTANGLAR BOX
;
setbox  proc    far
;
        push    si
        push    di
        push    dx
        push    bx              ; save registers
        push    cx
        push    ax
;
; determine byte position for start
;
; get y contribution
        mov     ax,y1           ; get starting y-coord
        mov     ah,al           ; replicate for odd/even bank
        and     ax,1FEh         ; just one bit gets moved
        sal     ax.1            ; times 4
        sal     ax,1            ; times 8
        sal     ax,1            ; times 16
        mov     di,ax           ; address gets 16 times y-coordinate
        and     di,7FFh         ; not the odd/even bit
        sal     ax,1            ; times 32
        sal     ax,1            ; times 64
        add     di,ax           ; address gets y-coord times 80
;
; add in x contribution
        mov     ax,xl           ; get x-coordinate
        sar     ax,1            ; divide
        sar     ax,1            ;  by 4
        add     di,ax           ; beginning offset
;
; count for outer loop
        mov     cx,y2           ; ending y-coordinate
        sub     cx,y1           ; minus starting y-coord
        inc     cx              ; plus one
;
; count for inner loop
        mov     si,x2           ; ending x-coordinate
        sar     si,1            ; divide
        sar     si,1            ;  by 4
        mov     ax,x1           ; starting x-coordinate
        sar     ax,1            ; divide
        sar     ax,1            ;  by 4
        sub     si,ax           ; take the difference
;
; get the color
        mov     bx,color        ; get the color
        and     bx,3            ; just between 0 and 3
        mov     dl,cbytes[bx]   ; look up color pattern
;
; determine mask for start and ending bytes
        mov     bx,xl           ; starting byte
        and     bx,3            ; just the pixel position
        sal     bx,1            ; times 2
        sal     bx,1            ; times 4
        mov     ax,x2           ; ending byte
        and     ax,3            ; just the pixel position
        add     bx,ax           ; 4*starting+ending
        sal     bx,1            ; 8*starting+2*ending
        mov     bx,xtable[bx]   ; look up the masks
;
; set up masked color bytes
        mov     dh,dl           ; color for left bytes
        mov     ah,dl           ; color for middle bytes
        and     dx,bx           ; mask left and right color bytes
;
        cld                     ; forward
;
sboxloop:
        push    cx              ; save count of outer loop
        push    di              ; save initial byte position
;
        mov     cx,si   ; count for inner loop
;
;check for only one byte
        mov     al,bh           ; get the mask
        jcxz    sboxloop2       ; if ending byte coincides
;
; color leftmost byte of the scan line
        not     al              ; reverse the mask for clearing
        and     al,es:[di]      ; get byte from memory and clear pixels
        or      al,dh           ; put color in place
        stosb                   ; put byte in place
;
; check for just two bytes
        dec     cx              ; count the byte
        jcxz    sboxloop1       ; done?
;
; color the middle bytes of the scan line
        mov     al,ah           ; color for middle bytes
        rep     stosb           ; put middle bytes in place
;
; handle rightmost byte of the scan line
;
; come here if two or more bytes
sboxloop1:
        mov     al,0FFh         ; set full mask
;
; in any case come here ro adjust the masks
sboxloop2:
        and     al,bl           ; bring in right part of mask
        and     dl,al           ; clear left part of color if needed
;
; color the byte
        not     al              ; reverse the mask for clearing
        and     al,es:[di]      ; get byte from memory and clear pixels
        or      al,dl           ; put pixels in the byte
        stosb
;
; compute next scan line
        pop     di              ; restore address of left side of box
        test    di,2000h        ; odd or even line?
        jz      sboxloop3       ; skip if even
        add     di,80           ; add 80 bytes per line
sboxloop3:
        xor     di,2000h        ; changes banks in any case
        pop     cx              ; restore count for outer loop
        loop    sboxloop        ; next scan line
;
        pop     ax              ; restore registers
        pop     cx
        pop     bx
        pop     dx
        pop     di
        pop     si
        ret
;
setbox  endp

